# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import django

from django.db import IntegrityError, models
from django.template.defaultfilters import slugify as default_slugify
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _

from parler import appsettings
from parler.managers import TranslatableManager, TranslatableQuerySet
from parler.models import TranslatableModel, TranslatedFields
from treebeard.ns_tree import NS_Node, NS_NodeManager, NS_NodeQuerySet

LANGUAGE_CODES = appsettings.PARLER_LANGUAGES.get_active_choices()


class CategoryQuerySet(TranslatableQuerySet, NS_NodeQuerySet):
    pass


class CategoryManager(TranslatableManager, NS_NodeManager):
    queryset_class = CategoryQuerySet

    def get_queryset(self):
        return self.queryset_class(
            self.model,
            using=self._db
        ).order_by('tree_id', 'lft')

    if django.VERSION < (1, 8):
        get_query_set = get_queryset


#
# TODO: I would have preferred to make this a base class "CategoryBase" which
# is Abstract, then subclass it as a concrete Category class. But, Parler
# cannot be applied to an Abstract class.
#
# TODO: At some point, consider an approach like this:
#     https://gist.github.com/GaretJax/7c7a9acc055c05c65041
#
@python_2_unicode_compatible
class Category(TranslatableModel, NS_Node):
    """
    A category is hierarchical. The structure is implemented with django-
    treebeard's Nested Sets trees, which has the performance characteristics
    we're after, namely: fast reads at the expense of write-speed.
    """

    translations = TranslatedFields(
        name=models.CharField(
            _('name'),
            blank=False,
            default='',
            max_length=255,
        ),
        slug=models.SlugField(
            _('slug'),
            blank=True,
            default='',
            help_text=_('Provide a “slug” or leave blank for an automatically '
                        'generated one.'),
            max_length=255,
        ),
        meta={'unique_together': (('language_code', 'slug', ), )}
    )

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    objects = CategoryManager()

    def slugify(self, category, i=None):
        slug = default_slugify(category)
        if i is not None:
            slug += "_%d" % i
        return slug

    def save(self, *args, **kwargs):
        if not self.slug:
            self.slug = self.slugify(self.name)
            try:
                return super(Category, self).save(*args, **kwargs)
            except IntegrityError:
                pass

            for lang in LANGUAGE_CODES:
                #
                # We'd much rather just do something like:
                #     Category.objects.translated(lang,
                #         slug__startswith=self.slug)
                # But sadly, this isn't supported by Parler/Django, see:
                #     http://django-parler.readthedocs.org/en/latest/api/\
                #         parler.managers.html#the-translatablequeryset-class
                #
                slugs = []
                all_slugs = Category.objects.language(lang).exclude(
                    id=self.id).values_list('translations__slug', flat=True)
                for slug in all_slugs:
                    if slug and slug.startswith(self.slug):
                        slugs.append(slug)
                i = 1
                while True:
                    slug = self.slugify(self.name, i)
                    if slug not in slugs:
                        self.slug = slug
                        return super(Category, self).save(*args, **kwargs)
                    i += 1
        else:
            return super(Category, self).save(*args, **kwargs)

    def delete(self, using=None):
        #
        # We're simply managing how the two superclasses perform deletion
        # together here.
        #
        self.__class__.objects.filter(pk=self.pk).delete(using)
        super(TranslatableModel, self).delete()

    def __str__(self):
        return self.safe_translation_getter('name', any_language=True)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals
# import warnings

from django.core.exceptions import ImproperlyConfigured
from django.db.models.fields.related import (
    ForeignKey,
    ManyToManyField,
    OneToOneField
)
from django.forms.models import ModelChoiceField, ModelMultipleChoiceField
from django.utils.safestring import mark_safe

# For South, where used.
try:
    from south.modelsinspector import add_introspection_rules
except:  # pragma: no cover
    add_introspection_rules = False

from .models import Category


class CategoryModelChoiceField(ModelChoiceField):
    """Displays choices hierarchically as per their position in the tree."""
    def label_from_instance(self, obj):
        prefix = ''
        try:
            if obj.depth > 1:
                prefix = '&nbsp;&nbsp;' * (obj.depth - 1)

            return mark_safe("{prefix}{name}".format(
                prefix=prefix, name=obj.safe_translation_getter('name')
            ))
        except AttributeError:
            raise ImproperlyConfigured(
                "CategoryModelChoiceField should only be used for ForeignKey "
                "relations to the aldryn_categories.Category model.")


class CategoryForeignKey(ForeignKey):
    """
    Simply a normal ForeignKey field, but with a custom *default* form field
    which hierarchically displays the set of choices.
    """

    def __init__(self, to=Category, **kwargs):
        """Sets Category as the default `to` parameter."""
        super(CategoryForeignKey, self).__init__(to, **kwargs)

    # This is necessary for Django 1.7.4+
    def get_internal_type(self):
        return 'ForeignKey'

    def formfield(self, form_class=CategoryModelChoiceField,
                  choices_form_class=None, **kwargs):
        kwargs["form_class"] = form_class
        kwargs["choices_form_class"] = choices_form_class
        return super(CategoryForeignKey, self).formfield(**kwargs)


class CategoryOneToOneField(OneToOneField):
    """
    Simply a normal OneToOneField field, but with a custom *default* form field
    which hierarchically displays the set of choices.
    """

    def __init__(self, to=Category, **kwargs):
        """Sets Category as the default `to` parameter."""
        super(CategoryOneToOneField, self).__init__(to, **kwargs)

    # This is necessary for Django 1.7.4+
    def get_internal_type(self):
        return 'ForeignKey'

    def formfield(self, form_class=CategoryModelChoiceField,
                  choices_form_class=None, **kwargs):
        kwargs["form_class"] = form_class
        kwargs["choices_form_class"] = choices_form_class
        return super(OneToOneField, self).formfield(**kwargs)


class CategoryMultipleChoiceField(ModelMultipleChoiceField):
    """Displays choices hierarchically as per their position in the tree."""
    def label_from_instance(self, obj):
        prefix = ''
        try:
            if obj.depth > 1:
                prefix = '&nbsp;&nbsp;' * (obj.depth - 1)

            return mark_safe("{prefix}{name}".format(
                prefix=prefix, name=obj.safe_translation_getter('name')
            ))
        except AttributeError:
            raise ImproperlyConfigured(
                "CategoryMultipleChoiceField should only be used for M2M "
                "relations to the aldryn_categories.Category model.")


class CategoryManyToManyField(ManyToManyField):
    """
    Simply a normal ManyToManyField, but with a custom *default* form field
    which hierarchically displays the set of choices.
    """

    def __init__(self, to=Category, **kwargs):
        """Sets Category as the default `to` parameter."""
        super(CategoryManyToManyField, self).__init__(to, **kwargs)

    # This is necessary for Django 1.7.4+
    def get_internal_type(self):
        return 'ManyToManyField'

    def formfield(self, form_class=CategoryMultipleChoiceField,
                  choices_form_class=None, **kwargs):
        kwargs["form_class"] = form_class
        kwargs["choices_form_class"] = choices_form_class
        return super(CategoryManyToManyField, self).formfield(**kwargs)


# This is necessary for South
if add_introspection_rules:
    add_introspection_rules([], [
        "^aldryn_categories\.fields\.CategoryForeignKey",
        "^aldryn_categories\.fields\.CategoryManyToManyField",
        "^aldryn_categories\.fields\.CategoryOneToOneField"
    ])
